#!/usr/bin/perl

# (C) Maxim Dounin

# Tests for http proxy cache revalidation with conditional requests.

###############################################################################

use warnings;
use strict;

use Test::More;

BEGIN { use FindBin; chdir($FindBin::Bin); }

use lib 'lib';
use Test::Nginx qw/ :DEFAULT :gzip /;

###############################################################################

select STDERR; $| = 1;
select STDOUT; $| = 1;

my $t = Test::Nginx->new()->has(qw/http proxy cache rewrite shmem/)->plan(16)
        ->write_file_expand('nginx.conf', <<'EOF');

%%TEST_GLOBALS%%

daemon off;

events {
}

http {
    %%TEST_GLOBALS_HTTP%%

    proxy_cache_path   %%TESTDIR%%/cache  levels=1:2
                       keys_zone=one:1m;

    proxy_cache_revalidate on;

    server {
        listen       127.0.0.1:8080;
        server_name  localhost;

        location / {
            proxy_pass    http://127.0.0.1:8081;
            proxy_cache   one;
            add_header X-Cache-Status $upstream_cache_status;
        }
    }

    server {
        listen       127.0.0.1:8081;
        server_name  localhost;

        location / { }
        location /etag/ {
            proxy_pass http://127.0.0.1:8081/;
            proxy_hide_header Last-Modified;
        }
        location /l-m/ {
            proxy_pass http://127.0.0.1:8081/;
            proxy_hide_header ETag;
        }
        location /etag-0/ {
            proxy_pass http://127.0.0.1:8081/;
            add_header Cache-Control "max-age=0";
            proxy_hide_header Last-Modified;
        }
        location /l-m-0/ {
            proxy_pass http://127.0.0.1:8081/;
            add_header Cache-Control "max-age=0";
            proxy_hide_header ETag;
        }
        location /0/ {
            proxy_pass http://127.0.0.1:8081/;
            add_header Cache-Control "max-age=0";
            proxy_hide_header ETag;
            proxy_hide_header Last-Modified;
        }
        location /bad-l-m-0/ {
            proxy_pass http://127.0.0.1:8081/;
            add_header Cache-Control "max-age=0";
            proxy_hide_header ETag;
            proxy_hide_header Last-Modified;
            add_header Last-Modified "bad";
        }
        location /auth/ {
            if ($http_x_secret = "") {
                return 401;
            }
            if ($http_x_secret = "0") {
                return 403;
            }
            proxy_pass http://127.0.0.1:8081/;
            add_header Cache-Control "max-age=0";
        }
    }
}

EOF

my $d = $t->testdir();

$t->write_file('t', 'SEETHIS');

$t->run();

###############################################################################

# check various combinations of max-age=0 and ETag/Last-Modified headers

my $CACHE_MISS = qr/X-Cache-Status: MISS.*?SEETHIS/ms;
my $REVALIDATED = qr/X-Cache-Status: REVALIDATED.*?SEETHIS/ms;

like(http_get('/etag-0/t'), $CACHE_MISS, 'has ETag and max-age=0');
like(http_get('/etag-0/t'), $REVALIDATED, '... revalidated');

like(http_get('/l-m-0/t'), $CACHE_MISS, 'has Last-Modified and max-age=0');
like(http_get('/l-m-0/t'), $REVALIDATED, '... revalidated');

like(http_get('/etag/t'), $CACHE_MISS, 'has ETag but no max-age');
like(http_get('/etag/t'), $CACHE_MISS, '... not cached');

like(http_get('/l-m/t'), $CACHE_MISS, 'has Last-Modified but no max-age');
like(http_get('/l-m/t'), $CACHE_MISS, '... not cached');

like(http_get('/0/t'), $CACHE_MISS, 'max-age=0 but no ETag or Last-Modified');
like(http_get('/0/t'), $CACHE_MISS, '... not cached');

like(http_get('/bad-l-m-0/t'), $CACHE_MISS, 'max-age=0 but bad Last-Modified');
like(http_get('/bad-l-m-0/t'), $CACHE_MISS, '... not cached');

# check that authorized requests are cached and revalidated (the whole point)

sub http_get_secret {
    my ($url, $secret) = @_;
    $secret = int($secret);

    return http(<<EOF);
GET $url HTTP/1.0
Host: localhost
X-Secret: $secret

EOF
}

like(http_get_secret('/auth/t', 1), $CACHE_MISS, 'user 1 cached');
like(http_get('/auth/t'), qr/ 401 /, 'no user unauthorized');
like(http_get_secret('/auth/t', 0), qr/ 403 /, 'user 0 forbidden');
like(http_get_secret('/auth/t', 2), $REVALIDATED, 'user 2 revalidated');
